//utf8

/*
 * Copyright 2007 (C) Michal Kalewski
 * mkalewski@cs.put.poznan.pl
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the MIT License.
 * You should have received a copy of the MIT License along with this
 * software; if not, you can download a copy from http://www.opensource.org.
 *
 * For questions, bug reports, feature requests, support requests etc.
 * send an e-mail to: mkalewski@cs.put.poznan.pl.
 *
 * $Id: smscomm.cpp,v 1.13 2008-01-11 14:27:48 mkalewski Exp $
 *
 */

#include "smscomm.h"

//Logger class

Logger::Logger(bool bQuiet, bool bDebug) {
        _bQuiet = bQuiet;
        _bDebug = bDebug;
        if (_bQuiet == true) _bDebug = false;
        bzero(&_cLastMsg, sizeof(_cLastMsg));
}

void Logger::Print(const char* pcText, const char* pcLevel) {
        char buf[30];
        struct timeval tv;
        time_t curtime;

        gettimeofday(&tv, NULL);
        curtime=tv.tv_sec;
        strftime(buf, 30, "%m/%d/%Y %T", localtime(&curtime));
        sprintf(_cLastMsg, "[%s] %-7s %s\n", buf, pcLevel, pcText);
        if (strncmp(pcLevel, "ERROR", 5) != 0)
                fprintf(stdout, _cLastMsg);
        else
                fprintf(stderr, _cLastMsg);
}

void Logger::Log(const char* pcText) {
        if (_bQuiet == false) Print(pcText, "INFO");
}

void Logger::Info(const char* pcText) {
        if (_bQuiet == false) Print(pcText, "INFO");
}

void Logger::Debug(const char* pcText) {
        if (_bDebug == true) Print(pcText, "DEBUG");
}

void Logger::Warning(const char* pcText) {
        if (_bQuiet == false) Print(pcText, "WARNING");
}

void Logger::Error(const char* pcText) {
        if (_bQuiet == false) Print(pcText, "ERROR");
}

void Logger::GetLastMsg(char* pcMSG, unsigned short int usiMaxLength) {
        if ((strlen(_cLastMsg)+1) <= usiMaxLength)
                strncpy(pcMSG, _cLastMsg, strlen(_cLastMsg)+1);
        else
                strncpy(pcMSG, _cLastMsg, usiMaxLength);
}

//SerialPort class

SerialPort::SerialPort(Logger* pLogger) {
        strcpy(_pcSerialPortName, SERIALPORTNAME);
        _pLogger = pLogger;
}

SerialPort::SerialPort(const char* pcSerialPortName, Logger* pLogger) {
	if ((strlen(pcSerialPortName)+1) <= sizeof(_pcSerialPortName))
                strcpy(_pcSerialPortName, pcSerialPortName);
        _pLogger = pLogger;
}

SerialPort::~SerialPort() {
        if (IsOpen())
                close(_iSerialPort);
        _pLogger->Debug("Serial port is closed");
}

bool SerialPort::IsOpen() {
        return (_iSerialPort > 0 ) ? true : false;
}

void SerialPort::SerialPortSetings() {
	struct termios options;

        bzero(&options, sizeof(options));
	tcgetattr(_iSerialPort, &options);
	options.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
        options.c_iflag = IGNPAR | ICRNL | IGNCR;
        options.c_oflag = 0;
        options.c_lflag = ICANON;
        tcflush(_iSerialPort, TCIFLUSH);
        tcsetattr(_iSerialPort, TCSANOW, &options);
        _pLogger->Debug("Serial port is ready");
}

bool SerialPort::Open() {
        int flags = 0;
        char logbuf[255];

	_iSerialPort = open(_pcSerialPortName, O_RDWR | O_NOCTTY);
	if (_iSerialPort >= 0) {
                fcntl(_iSerialPort, F_SETFL, 0);
                _pLogger->Debug("Serial port is opened");
                SerialPortSetings();
                return true;
	}
        bzero(&logbuf, sizeof(logbuf));
        sprintf(logbuf, "Open(): Unable to open the given port! \"%s\"", strerror(errno));
        _pLogger->Error(logbuf);
        return false;
}

bool SerialPort::Write(const char* pcText) {
        char buf[255];
        char logbuf[255];

	if (IsOpen()) {
		int foo = write(_iSerialPort, pcText, strlen(pcText));

		if (foo > 0) {
                        bzero(&buf, sizeof(buf));
                        bzero(&logbuf, sizeof(logbuf));
                        strcpy(buf, pcText);
                        buf[strlen(buf)-1] = '\0';
                        sprintf(logbuf, "Write[%d]: \"%s\"", foo, buf);
                        _pLogger->Debug(logbuf);
			return true;
                }
                bzero(&buf, sizeof(buf));
                bzero(&logbuf, sizeof(logbuf));
                strcpy(buf, pcText);
                buf[strlen(buf)-1] = '\0';
                sprintf(logbuf, "Write(): Write[%d]: \"%s\"! \"%s.\"", foo, buf, strerror(errno));
                _pLogger->Error(logbuf);
                return false;
	}
        _pLogger->Error("Write(): Serial port NOT ready!");
	return false;
}

bool SerialPort::Read(char* pcBuf, unsigned short int usiMaxLength) {
        int  nbytes = -1;
        char buf[255];
        char logbuf[255];

        bzero(&buf, sizeof(buf));
        bzero(&logbuf, sizeof(logbuf));
        if (IsOpen()) {
                nbytes = read(_iSerialPort, buf, sizeof(buf));
                if (nbytes > 0) buf[nbytes - 1] = '\0';
                sprintf(logbuf, "Read: [%d] \"%s\"", nbytes, buf);
                _pLogger->Debug(logbuf);
                if (nbytes > 0) {
                        if (nbytes < usiMaxLength)
                                strncpy(pcBuf, buf, nbytes);
                        else
                                strncpy(pcBuf, buf, usiMaxLength);
                        return true;
                }
                return false;
        }
        _pLogger->Error("Read(): Serial port NOT ready!");
        return false;
}

//SMSRec class

SMSRec::SMSRec() {
        _siSMSNumber = -1;
        bzero(&_pcSMSStatus, sizeof(_pcSMSStatus));
        bzero(&_pcSMSGSMNumber, sizeof(_pcSMSGSMNumber));
        bzero(&_pcSMSSendTime, sizeof(_pcSMSSendTime));
        bzero(&_pcSMSText, sizeof(_pcSMSText));
}

bool SMSRec::GetSMS(char* pcSMS, unsigned short int usiMaxLength) {
        char buf[1024];

        if (_siSMSNumber >= 0) {
                bzero(&buf, sizeof(buf));
                sprintf(buf, "%s\t%s\t%s\n%s\n\n", _pcSMSStatus, _pcSMSGSMNumber, _pcSMSSendTime, _pcSMSText);
                if ((strlen(buf)+1) <= usiMaxLength)
                        strncpy(pcSMS, buf, strlen(buf)+1);
                else
                        strncpy(pcSMS, buf, usiMaxLength);
                return true;
        }
        return false;
}

void SMSRec::PrintSMS() {
        printf("%s\t%s\t%s\n%s\n", _pcSMSStatus, _pcSMSGSMNumber, _pcSMSSendTime, _pcSMSText);
}

//GSMModem class

GSMModem::GSMModem(Logger* pLogger) : SerialPort(pLogger) {
        StartupDevice();
}

GSMModem::GSMModem(const char* pcSerialPortName, Logger* pLogger) : SerialPort(pcSerialPortName, pLogger) {
        StartupDevice();
}

GSMModem::~GSMModem() {
        if (IsReady()) {
                if (Write(DEFAULTCONFIGURATION) && Response(_OK)) {
                        _pLogger->Log("Serial port line and device are closed");
                }
                else
                        _pLogger->Warning("~GSMModem(): Cannot set default device configuration!");
        }
        else
                _pLogger->Error("~GSMModem(): Device NOT ready!");
}

bool GSMModem::StartupDevice() {
        bool result;

        _bDevicePIN = false;
        _bDevice = false;
        if (DELETEREADSMS) _pLogger->Log("Deleting read SMS enabled");
        if (Open()) {
                if (Write(REQUESTIDENTIFICATION) && Response(_OK)) {
                        _pLogger->Debug("Device is ready");
                        if (Write(DEFAULTCONFIGURATION) && Response(_OK)) {
                                if (Write(ECHOOFF) && Response(_OK)) {
                                        _bDevice = true;
                                        _pLogger->Debug("Device is initialized");
                                        _pLogger->Log("Device is initialized and ready to use");
                                        return true;
                                }
                                _pLogger->Error("StartupDevice(): Cannot set echo off!");
                        }
                        else
                                _pLogger->Error("StartupDevice(): Cannot set default device configuration!");
                }
                else
                        _pLogger->Error("StartupDevice(): Cannot get device identification!");
        }
        _pLogger->Error("StartupDevice(): Device NOT initialized!");
        return false;
}

bool GSMModem::IsReady() {
        return _bDevice;
}

bool GSMModem::IsReadyPIN() {
        return (_bDevicePIN && _bDevice);
}

bool GSMModem::ATErrorsHandler(const char* pcText) {
        char logbuf[255];

        bzero(&logbuf, sizeof(logbuf));
        if (strncmp(pcText, __ERROR1, strlen(__ERROR1)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __ERROR1);
                _pLogger->Error(logbuf);
                return false;
        }
        if (strncmp(pcText, __ERROR2, strlen(__ERROR2)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __ERROR2);
                _pLogger->Error(logbuf);
                return false;
        }
        if (strncmp(pcText, __CMEERROR1, strlen(__CMEERROR1)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __CMEERROR1);
                _pLogger->Error(logbuf);
                return false;
        }
        if (strncmp(pcText, __CMEERROR2, strlen(__CMEERROR2)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __CMEERROR2);
                _pLogger->Error(logbuf);
                return false;
        }
        if (strncmp(pcText, __CMSERROR1, strlen(__CMSERROR1)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __CMSERROR1);
                _pLogger->Error(logbuf);
                return false;
        }
        if (strncmp(pcText, __CMSERROR2, strlen(__CMSERROR2)) == 0) {
                sprintf(logbuf, "%s \"%s\"", "Response(): Device error response!", __CMSERROR2);
                _pLogger->Error(logbuf);
                return false;
        }
        return true;
}

bool GSMModem::Response(const char* pcResponse) {
        char buf[255];
        char logbuf[255];

        bzero(&buf, sizeof(buf));
        bzero(&logbuf, sizeof(logbuf));
        while(Read(buf, sizeof(buf))) {
                if (strlen(buf) > 1) {
                        if (strncmp(pcResponse, buf, strlen(pcResponse)) == 0) {
                                if ((strncmp(pcResponse, _REGISTERNETWORK1, strlen(_REGISTERNETWORK1))) == 0)
                                        Response(_OK);
                                return true;
                        }
                        if (strncmp(pcResponse, _REGISTERNETWORK1, strlen(pcResponse)) == 0) {
                                if (strncmp(_REGISTERNETWORK2, buf, strlen(_REGISTERNETWORK2)) == 0) {
                                        Response(_OK);
                                        return false;
                                }
                        }
                        if (!ATErrorsHandler(buf))
                                return false;
                }
                bzero(&buf, sizeof(buf));
                bzero(&logbuf, sizeof(logbuf));
        }
        return false;
}

void GSMModem::SetSMSHeader(const char* pcSMSText, SMSRec* pSMS) {
        char delims[] = "\"";
        char* result = NULL;
        char buf[255];
        unsigned short int foo = 0;

        if (strlen(pcSMSText) <= sizeof(buf)) {
                bzero(&buf, sizeof(buf));
                strncpy(buf, pcSMSText, strlen(pcSMSText)+1);
                result = strtok(buf, delims);
                while(result != NULL) {
                        switch (foo) {
                                case 1: if (strlen(result)+1 <= sizeof(pSMS->_pcSMSStatus)) {
                                                strncpy(pSMS->_pcSMSStatus, result, strlen(result));
                                                pSMS->_pcSMSStatus[strlen(result)] = '\0';
                                                _pLogger->Debug("SMS: status set");
                                        }
                                        break;
                                case 3: if (strlen(result)+1 <= sizeof(pSMS->_pcSMSGSMNumber)) {
                                                strncpy(pSMS->_pcSMSGSMNumber, result, strlen(result));
                                                pSMS->_pcSMSGSMNumber[strlen(result)] = '\0';
                                                _pLogger->Debug("SMS: GSM number set");
                                        }
                                        break;
                                case 5:
                                case 7: if (strlen(result)+1 <= sizeof(pSMS->_pcSMSSendTime)) {
                                                strncpy(pSMS->_pcSMSSendTime, result, strlen(result));
                                                pSMS->_pcSMSSendTime[strlen(result)] = '\0';
                                                _pLogger->Debug("SMS: time and date set");
                                        }
                                        break;
                        }
                        foo++;
                        result = strtok(NULL, delims);
                }
        }
}

void GSMModem::PrintVersion() {
        printf("%s\n%s\n\n", VERSION, COPYRIGHT);
}

bool GSMModem::EnterPIN(const char* pcPIN) {
        char buf[255];

        if (IsReady()) {
                if (Write(REGISTERNETWORK) && Response(_REGISTERNETWORK1)) {
                        _pLogger->Log("PIN already entered");
                        _bDevicePIN = true;
                        return true;
                }
                if (strlen(pcPIN) == 4) {
                        bzero(&buf, sizeof(buf));
                        sprintf(buf, "%s%s\r", ENTERPIN, pcPIN);
                        if (Write(buf) && Response(_OK)) {
                                _pLogger->Debug("PIN entered");
                                _pLogger->Log("Device logged in (PIN OK)");
                                _bDevicePIN = true;
                                return true;
                        }
                        _pLogger->Error("EnterPIN(): Cannot set PIN!");
                        _bDevicePIN = false;
                        return false;
                }
                _pLogger->Error("EnterPIN(): Wrong PIN!");
                _bDevicePIN = false;
                return false;
        }
        _pLogger->Error("EnterPIN(): Device NOT ready!");
        return false;
}

bool GSMModem::SendSMS(const char* pcGSMNumber, const char* pcSMSText) {
        int foo = 0, length = 0;
        char buf[255];

        length = strlen(pcGSMNumber);
        if (IsReadyPIN()) {
                if (length == 0) {
                        _pLogger->Error("SendSMS(): Empty GSM number!");
                        return false;
                }
                if (strlen(pcSMSText) == 0) {
                        _pLogger->Error("SendSMS(): Empty SMS text!");
                        return false;
                }
                if (strlen(pcSMSText) > MAXSMSLENGTH) {
                        _pLogger->Error("SendSMS(): SMS text is too long!");
                        return false;
                }
                for (foo = 0; foo < length; foo++) {
                        if (pcGSMNumber[foo] == '+') continue;
                        if (pcGSMNumber[foo] == '0') continue;
                        if (pcGSMNumber[foo] == '1') continue;
                        if (pcGSMNumber[foo] == '2') continue;
                        if (pcGSMNumber[foo] == '3') continue;
                        if (pcGSMNumber[foo] == '4') continue;
                        if (pcGSMNumber[foo] == '5') continue;
                        if (pcGSMNumber[foo] == '6') continue;
                        if (pcGSMNumber[foo] == '7') continue;
                        if (pcGSMNumber[foo] == '8') continue;
                        if (pcGSMNumber[foo] == '9') continue;
                        _pLogger->Error("SendSMS(): Wrong GSM number!");
                        return false;
                }
                bzero(&buf, sizeof(buf));
                sprintf(buf, "%s%s", "Sending SMS to ", pcGSMNumber);
                _pLogger->Debug(buf);
                if (Write(TEXTMODE) && Response(_OK)) {
                        _pLogger->Debug("Text mode set");
                        bzero(&buf, sizeof(buf));
                        sprintf(buf, "%s\"%s\"\r", SENDMESSAGE, pcGSMNumber);
                        if (Write(buf)) {
                                _pLogger->Debug("GSM number set");
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "%s%s\032", pcSMSText, SMSFOOTER);
                                if (Write(buf) && Response(_OK)) {
                                        bzero(&buf, sizeof(buf));
                                        sprintf(buf, "%s%s", "SMS sent to ", pcGSMNumber);
                                        _pLogger->Log(buf);
                                        return true;
                                }
                                _pLogger->Error("SendSMS(): Cannot send SMS!");
                                return false;
                        }
                        _pLogger->Error("SendSMS(): Cannot set GSM number!");
                        return false;
                }
                _pLogger->Error("SendSMS(): Cannot set text mode!");
                return false;
        }
        _pLogger->Error("SendSMS(): Device NOT ready or NOT logged in!");
        return false;
}

bool GSMModem::EraseSMS(unsigned short int usiSMSNumber) {
        char buf[255];

        if (IsReadyPIN()) {
                bzero(&buf, sizeof(buf));
                sprintf(buf, "%s%d", "Deleting SMS number ", usiSMSNumber);
                _pLogger->Debug(buf);
                bzero(&buf, sizeof(buf));
                sprintf(buf, "%s%d\r", ERASEMESSAGE, usiSMSNumber);
                if (Write(buf) && Response(_OK)) {
                        bzero(&buf, sizeof(buf));
                        sprintf(buf, "%s%d%s", "SMS number ", usiSMSNumber, " deleted");
                        _pLogger->Log(buf);
                        return true;
                }
                sprintf(buf, "%s%d%s", "EraseSMS(): Cannot delete SMS number ", usiSMSNumber, "!");
                _pLogger->Log(buf);
                return false;
        }
        _pLogger->Error("EraseSMS(): Device NOT ready or NOT logged in!");
        return false;
}

bool GSMModem::ReceiveSMS(unsigned short int usiSMSNumber, SMSRec* pSMS) {
        char buf[255];

        if (IsReadyPIN()) {
                if (pSMS == NULL) {
                        _pLogger->Error("ReceiveSMS(): NULL pointer!");
                        return false;
                }
                sprintf(buf, "%s%d", "Receiving SMS number ", usiSMSNumber);
                _pLogger->Debug(buf);
                if (Write(TEXTMODE) && Response(_OK)) {
                        _pLogger->Debug("Text mode set");
                        bzero(&buf, sizeof(buf));
                        sprintf(buf, "%s%d\r", RECEIVEMESSAGE, usiSMSNumber);
                        if (Write(buf)) {
                                pSMS->_siSMSNumber = -1;
                                bzero(&buf, sizeof(buf));
                                while (Read(buf, sizeof(buf))) {
                                        if (strlen(buf) > 1) {
                                                if (!ATErrorsHandler(buf))
                                                        break;
                                                if (strncmp(buf, _RECEIVEMESSAGE, strlen(_RECEIVEMESSAGE)) == 0) {
                                                        SetSMSHeader(buf, pSMS);
                                                        pSMS->_siSMSNumber = usiSMSNumber;
                                                        bzero(&buf, sizeof(buf));
                                                        continue;
                                                }
                                                if (strncmp(buf, _OK, strlen(_OK)) == 0) {
                                                        if (pSMS->_siSMSNumber == usiSMSNumber) {
                                                                bzero(&buf, sizeof(buf));
                                                                sprintf(buf, "%s%d%s", "SMS number ", usiSMSNumber, " received");
                                                                _pLogger->Log(buf);
                                                                if (DELETEREADSMS)
                                                                        EraseSMS(usiSMSNumber);
                                                                return true;
                                                        }
                                                        else
                                                                break;
                                                }
                                                if ((strlen(buf) + strlen(pSMS->_pcSMSText) + 1) <= sizeof(pSMS->_pcSMSText)) {
                                                        if (strlen(pSMS->_pcSMSText))
                                                                strcat(pSMS->_pcSMSText, "\n");
                                                        strcat(pSMS->_pcSMSText, buf);
                                                }
                                        }
                                        bzero(&buf, sizeof(buf));
                                }
                        }
                        bzero(&buf, sizeof(buf));
                        sprintf(buf, "%s%d%s", "ReceiveSMS(): Cannot receive SMS number ", usiSMSNumber, "!");
                        _pLogger->Warning(buf);
                        return false;
                }
                _pLogger->Error("ReceiveSMS(): Cannot set text mode!");
                return false;
        }
        _pLogger->Error("ReceiveSMS(): Device NOT ready or NOT logged in!");
        return false;
}

bool GSMModem::GetFirstUnreadSMS(SMSRec* pSMS) {
        SMSRec* sms = new SMSRec();
        unsigned short int foo = 0;

        if (IsReadyPIN()) {
                if (pSMS == NULL) {
                        _pLogger->Error("GetFirstUnreadSMS(): NULL pointer!");
                        return false;
                }
                _pLogger->Log("Searching for first unread SMS");
                while(ReceiveSMS(foo, sms)) {
                        if ((strncmp(sms->_pcSMSStatus, ___UNREADMESSAGE, strlen(___UNREADMESSAGE)) == 0)) {
                                pSMS->_siSMSNumber = sms->_siSMSNumber;
                                strncpy(pSMS->_pcSMSStatus, sms->_pcSMSStatus, strlen(sms->_pcSMSStatus)+1);
                                strncpy(pSMS->_pcSMSGSMNumber, sms->_pcSMSGSMNumber, strlen(sms->_pcSMSGSMNumber)+1);
                                strncpy(pSMS->_pcSMSSendTime, sms->_pcSMSSendTime, strlen(sms->_pcSMSSendTime)+1);
                                strncpy(pSMS->_pcSMSText, sms->_pcSMSText, strlen(sms->_pcSMSText)+1);
                                return true;
                        }
                        foo++;
                        delete sms;
                        sms = new SMSRec();
                }
                _pLogger->Debug("No unread messages");
                return false;
        }
        _pLogger->Error("GetFirstUnreadSMS(): Device NOT ready or NOT logged in!");
        return false;
}

//EOF + NL
